# -*- coding: utf-8 -*-
from collections import OrderedDict

import requests
from requests.auth import HTTPBasicAuth

from . import exceptions

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin


TIMEOUT = 3


class ScrapydClient:
    def __init__(self, host, username=None, password=None):
        self.host = host

        if username is not None and password is not None:
            self.auth = HTTPBasicAuth(username, password)
        else:
            self.auth = None

    def get_status(self):
        """Get scrapyd status"""
        response = self.get('daemonstatus')
        self._assert_status_is_ok(response)

        return OrderedDict(response.items())

    def list_projects(self):
        """List projects uploaded to scrapyd"""
        response = self.get('listprojects')
        self._assert_status_is_ok(response)
        for project in response.get('projects', []):
            yield project

    def list_spiders(self, project):
        """List spiders for a project"""
        response = self.get('listspiders', project=project)

        try:
            self._assert_status_is_ok(response)
        except exceptions.ResponseNotOKException as e:
            if 'no active project' in str(e):
                raise exceptions.ProjectDoesNotExist('Project %s does not exist' % project)
            raise

        for spider in response.get('spiders', []):
            yield spider

    def schedule(self, project, spider):
        """Schedule a job for given project and spider"""
        response = self.post('schedule', data=dict(
            project=project,
            spider=spider,
        ))
        try:
            self._assert_status_is_ok(response)
        except exceptions.ResponseNotOKException as e:
            if 'no active project' in str(e):
                raise exceptions.ProjectDoesNotExist('Project %s does not exist' % project)

            if ': spider' in str(e) and str(e).endswith('not found'):
                raise exceptions.SpiderDoesNotExist('Spider %s does not exist' % spider)

            raise

        return response

    def get_log_link(self, project, spider, job):
        """Build a link to the log for a given project, spider and job"""
        return urljoin(self.host, '/'.join(['logs', project, spider, job]) + '.log')

    def _format_url(self, endpoint):
        """Append the API host"""
        return urljoin(self.host, '%s.json' % endpoint)

    def get(self, url, **kwargs):
        """Do a GET request"""
        r = requests.get(self._format_url(url), auth=self.auth, params=kwargs, timeout=TIMEOUT)
        self._assert_response_is_ok(r, 200)

        return r.json()

    def post(self, url, data, expected_status_code=200):
        """Do a POST request"""
        r = requests.post(self._format_url(url), data=data, auth=self.auth, timeout=TIMEOUT)
        self._assert_response_is_ok(r, expected_status_code)

        return r.json()

    def _assert_response_is_ok(self, response, expected_status_code):
        """Check sever response and raise exception if it is bad"""
        if response.status_code == 401:
            raise exceptions.UnAuthorizedException()

        if response.status_code != expected_status_code:
            raise exceptions.HTTPException('Got response code %d, expected %d, error: %s' % (response.status_code, expected_status_code, response.text))

    def _assert_status_is_ok(self, response):
        if 'status' not in response.keys():
            raise exceptions.ResponseNotOKException('Got bad server response: %s' % response)

        if response['status'] != 'ok':
            raise exceptions.ResponseNotOKException('Got non-ok server response: %s' % response.get('message', '<empty>'))



# -*- coding: utf-8 -*-
import pytest

from scrapyc import exceptions


def test_status(mocked_http_client, response):
    mocked_http_client.m.get('https://api.host.com:6800/daemonstatus.json', json=response('daemonstatus'))
    got = mocked_http_client.get_status()
    assert got['node_name'] == '8f3b385b77dc'


def test_list_projects(mocked_http_client, response):
    mocked_http_client.m.get('https://api.host.com:6800/listprojects.json', json=response('listprojects'))

    got = list(mocked_http_client.list_projects())
    assert len(got) == 11
    assert 'pik' in got


def test_list_projects_fail(mocked_http_client, response):
    json = response('listprojects')
    json['status'] = 'FAIL U R A L00ZER'
    mocked_http_client.m.get('https://api.host.com:6800/listprojects.json', json=json)

    with pytest.raises(exceptions.ResponseNotOKException):
        list(mocked_http_client.list_projects())


def test_list_spiders(mocked_http_client, response):
    mocked_http_client.m.get('https://api.host.com:6800/listspiders.json', json=response('listspiders'))

    got = list(mocked_http_client.list_spiders('pik'))
    assert got == ['spider1', 'spider2']


def test_bad_project_name(mocked_http_client, response):
    mocked_http_client.m.get('https://api.host.com:6800/listspiders.json', json=response('project_does_not_exist'))

    with pytest.raises(exceptions.ProjectDoesNotExist):
        list(mocked_http_client.list_spiders('nonexistant'))


def test_list_spiders_bad_response(mocked_http_client):
    """
    Check if non bad-project-name errors raise more common exception
    """
    mocked_http_client.m.get('https://api.host.com:6800/listspiders.json', json={'status': 'Я вас не знаю, идите нахер'})

    with pytest.raises(exceptions.ResponseNotOKException):
        list(mocked_http_client.list_spiders('stuff'))



from setuptools import find_packages, setup

setup(
    name='scrapyc',
    version='0.0.4',
    description="Simple client to scrapyd. Done right.",
    keywords=[],
    url="https://github.com/f213/scrapyd-client/",
    author="Fedor Borshev",
    author_email="f@f213.in",
    license="MIT",
    packages=find_packages(),
    install_requires=[
        'requests',
        'Click',
        'colorama',
    ],
    entry_points="""
        [console_scripts]
        scrapyc = scrapyc.cli:main
    """,
    include_package_data=True,
    zip_safe=False,
)



# -*- coding: utf-8 -*-
try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse

import click
import six

from scrapyc import ScrapydClient, exceptions


def validate_url(ctx, param, value):
    if value is None:
        raise click.UsageError('Please supply a crapyd URL (e.g. --url http://localhost:6800), or use SCRAPYC_URL environment variable')

    parsed = urlparse(value)
    if not len(parsed.scheme) or not len(parsed.netloc):
        raise click.BadParameter('Invalid scrapyd URL. Valid one should start with http:// or https://')

    return value


@click.group()
@click.option('--url', type=click.STRING, callback=validate_url, help='scrapyd URL')
@click.option('--username', type=click.STRING, help='(Optional) Username for basic authorization')
@click.option('--password', type=click.STRING, help='(Optional) Password for basic authorization')
@click.pass_context
def cli(ctx, url, username=None, password=None):
    ctx.obj['client'] = ScrapydClient(url, username, password)

    try:
        ctx.obj['client'].get_status()
    except exceptions.UnAuthorizedException:
        raise click.ClickException('Got 401 error during query to %s, please double-check username and password.' % url)


@cli.command(short_help='Run a project')
@click.argument('project', type=click.STRING)
@click.option('--spider', default='spider', type=click.STRING, help='(Optional) spider name')
@click.pass_context
def schedule(ctx, project, spider):
    try:
        response = ctx.obj['client'].schedule(project, spider)
    except exceptions.ProjectDoesNotExist:
        raise click.ClickException('Given project "%s" does not exist on %s' % (project, ctx.obj['client'].host))
    except exceptions.SpiderDoesNotExist:
        raise click.ClickException('Given spider "%s" is not present in the project %s' % (spider, project))

    log_link = ctx.obj['client'].get_log_link(project, spider, response['jobid'])

    click.secho('OK', fg='green', nl=False)
    click.echo(', %s' % log_link)


@cli.command(short_help='List spiders')
@click.argument('project', type=click.STRING)
@click.pass_context
def spiders(ctx, project):
    try:
        spiders = list(ctx.obj['client'].list_spiders(project))
    except exceptions.ProjectDoesNotExist:
        raise click.ClickException('Given project "%s" does not exist on %s' % (project, ctx.obj['client'].host))

    for spider in spiders:
        click.echo(spider)


@cli.command(short_help='List projects')
@click.pass_context
def projects(ctx):
    for project in ctx.obj['client'].list_projects():
        click.echo(project)


@cli.command(short_help='Get daemon status')
@click.pass_context
def status(ctx):
    status = ctx.obj['client'].get_status()

    node_name = status.pop('node_name')
    st = status.pop('status')
    click.echo('Scrapyd node name: ', nl=False)
    click.secho(node_name, fg='blue', nl=False)
    click.echo(', status: ', nl=False)
    click.secho(st.upper(), fg='green')
    click.echo()

    for key, value in six.iteritems(status):
        click.echo(key.title() + ': ', nl=False)
        click.secho(str(value), fg='green')


def main():
    return cli(
        obj=dict(),
        auto_envvar_prefix='SCRAPYC',
    )



# -*- coding: utf-8 -*-
import pytest

from scrapyc import exceptions


@pytest.mark.parametrize('input, formatted', [
    ('schedule', 'https://api.host.com:6800/schedule.json'),
    ('test/schedule', 'https://api.host.com:6800/test/schedule.json'),
    ('/schedule', 'https://api.host.com:6800/schedule.json'),
])
def test_format_url(input, formatted, client):
    assert client._format_url(input) == formatted


def test_get_log_link(client):
    assert client.get_log_link('project', 'spider', '100500') == 'https://api.host.com:6800/logs/project/spider/100500.log'


def test_get(mocked_http_client):
    mocked_http_client.m.get('https://api.host.com:6800/schedule.json', json={'status': 'ok'})
    assert mocked_http_client.get('schedule') == {'status': 'ok'}


def test_post(mocked_http_client):
    mocked_http_client.m.post('https://api.host.com:6800/schedule.json', json={'status': 'ok'}, status_code=299)
    assert mocked_http_client.post('schedule', data={'have': 'SPAM'}, expected_status_code=299) == {'status': 'ok'}


def test_http_auth(mocked_http_client):
    def check_auth(request, context):
        assert 'Authorization' in request.headers.keys()
        assert request.headers['Authorization'] == 'Basic cjAwdDpwYXNz'  # should be basic auth for r00t/pass
        return {
            'status': 'ok',
        }

    mocked_http_client.m.get('https://api.host.com:6800/schedule.json', json=check_auth)
    mocked_http_client.get('schedule')


def test_no_http_auth(mocked_http_client, monkeypatch):
    def check_no_auth(request, context):
        assert 'Authorization' not in request.headers.keys()
        return {
            'status': 'ok',
        }

    monkeypatch.setattr(mocked_http_client, 'auth', None)

    mocked_http_client.m.get('https://api.host.com:6800/schedule.json', json=check_no_auth)
    mocked_http_client.get('schedule')


def test_unauthorized(mocked_http_client):
    mocked_http_client.m.get('https://api.host.com:6800/schedule.json', status_code=401)
    with pytest.raises(exceptions.UnAuthorizedException):
        mocked_http_client.get('schedule')


def test_bad_server_response(mocked_http_client):
    mocked_http_client.m.get('https://api.host.com:6800/schedule.json', status_code=502)
    with pytest.raises(exceptions.HTTPException):
        mocked_http_client.get('schedule')
